package cz.upol.pja.lecture06;

import com.owlike.genson.Genson;
import java.io.InputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import javax.enterprise.context.ApplicationScoped;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.DELETE;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

@ApplicationScoped
@Path("/user")
public class UserList {
    
    private Map<Integer, User> users  = new HashMap<>();
    
    public UserList() {
        addUser(new User(1, "Tomas", "Marny"));
        addUser(new User(2, "Petr", "Klic"));
    }
    
    @GET
    @Path("list")
    @Produces(MediaType.APPLICATION_JSON)
    public synchronized Response list() {
        return Response.ok(users.values()).build();
    }
    
     
    // nefunugje v GlassFish 4.1.2
//    @POST
//    @Consumes(MediaType.APPLICATION_JSON)
//    @Path("create")
//    public synchronized Response create(User user) {
//        if (users.containsKey(user.getId())) 
//            return Response.status(Response.Status.CONFLICT).build();
//        users.put(user.getId(), user);
//        return Response.ok().build();
//    }
    
    // alternativni reseni vyuzivajici explicitni desiralizaci
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    public synchronized Response create(InputStream data) {
	Genson genson = new Genson();
	User user = genson.deserialize(data, User.class);
        
	if (users.containsKey(user.getId())) 
            return Response.status(Response.Status.CONFLICT).build();
	
        users.put(user.getId(), user);
        return Response.ok().build();
    }
    
    @GET
    @Path("{id}")
    public synchronized Response get(@PathParam("id") int id) {
        if (!users.containsKey(id))
            return Response.status(Response.Status.NOT_FOUND).build();
        return Response.ok(Collections.singleton(users.get(id)), MediaType.APPLICATION_JSON).build();
    }
    
    @PUT
    @Consumes(MediaType.APPLICATION_JSON)
    public synchronized Response put(User user) {
        if (!users.containsKey(user.id))
            return Response.status(Response.Status.NOT_FOUND).build();
	
        users.put(user.getId(), user);
        return Response.ok().build();
    }
    
    @DELETE
    @Path("{id}")
    public synchronized Response remove(@PathParam("id") int id) {
        if (users.remove(id) == null) 
            return Response.status(Response.Status.NOT_FOUND).build();
        return Response.ok().build();
    }
    
    
    private synchronized void addUser(User user) {
        users.put(user.getId(), user);
    }
    
    public static final class User {
        private int id;
        private String name;
        private String surname;
        
        public User() {
        }       

        public User(int id, String name, String surname) {
            this.id = id;
            this.name = name;
            this.surname = surname;
        }
        
        public int getId() {
            return id;
        }

        public void setId(int id) {
            this.id = id;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getSurname() {
            return surname;
        }

        public void setSurname(String surname) {
            this.surname = surname;
        }
    }
    
}
